@@ -6,10 +6,8 @@ from typing import Optional, Tuple
 import colorama
 import square
 import square.square
-from square import __version__
-from square.dtypes import (
-    DEFAULT_CONFIG_FILE, Config, Filepath, GroupBy, Selectors,
-)
+from square import DEFAULT_CONFIG_FILE, __version__
+from square.dtypes import Config, Filepath, GroupBy, Selectors
 
 # Convenience: global logger instance to avoid repetitive code.
 logit = logging.getLogger("square")
@@ -180,40 +178,35 @@ def compile_config(cmdline_param) -> Tuple[Config, bool]:
     # Load the default configuration unless the user specified an explicit one.
     if p.configfile:
         logit.info(f"Loading configuration file <{p.configfile}>")
-        cfg, err = square.square.load_config(p.configfile)
+        cfg, err = square.cfgfile.load(p.configfile)
 
-        # Use the folder the `load_config` function determined.
-        folder = cfg.folder
-
-        # Look for `--kubeconfig`. Default to the value in config file.
+        # Look for `--kubeconfig`. Defaults to the value in config file.
         kubeconfig = p.kubeconfig or cfg.kubeconfig
     else:
-        # Pick the configuration file, depending on whether the user specified
-        # `--no-config`, `--config` and whether a `.square.yaml` file exists.
+        # Pick the configuration file. Depends on whether the user specified
+        # `--no-config`, `--config` and if `.square.yaml` exists.
         default_cfg = DEFAULT_CONFIG_FILE
         dot_square = Filepath(".square.yaml")
+
         if p.no_config:
             cfg_file = default_cfg
         else:
             cfg_file = dot_square if dot_square.exists() else default_cfg
 
         logit.info(f"Loading configuration file <{cfg_file}>")
-        cfg, err = square.square.load_config(cfg_file)
+        cfg, err = square.cfgfile.load(cfg_file)
 
         # Determine which Kubeconfig to use. The order is: `--kubeconfig`,
         # `--config`, `.square`, `KUBECONFIG` environment variable.
         if cfg_file == default_cfg:
             kubeconfig = p.kubeconfig or os.getenv("KUBECONFIG", "")
+            cfg.folder = Filepath.cwd()
         else:
             kubeconfig = p.kubeconfig or str(cfg.kubeconfig) or os.getenv("KUBECONFIG", "")  # noqa
+
         del dot_square, default_cfg
 
-        # Use the current working directory as the folder directory because the
-        # user did not specify an explicit configuration file which would
-        # contain the desired folder.
-        folder = Filepath.cwd()
-
-        # Abort if neither `--kubeconfig` nor KUBECONFIG env var.
+        # Abort if neither `--kubeconfig` nor the KUBECONFIG environment variable exist.
         if not kubeconfig:
             logit.error("Must specify a Kubernetes config file.")
             return err_resp
@@ -221,7 +214,7 @@ def compile_config(cmdline_param) -> Tuple[Config, bool]:
         return err_resp
 
     # Override the folder if user specified "--folder".
-    folder = folder if p.folder is None else p.folder
+    folder = cfg.folder if p.folder is None else p.folder
 
     # Expand the user's home folder, ie if the path contains a "~".
     folder = Filepath(folder).expanduser()
