@@ -7,14 +7,14 @@ from typing import (
     Collection, DefaultDict, Dict, Iterable, List, Optional, Tuple,
 )
 
+import square.cfgfile
 import square.dotdict
 import square.k8s
-import square.schemas
 import yaml
 import yaml.scanner
 from square.dtypes import (
-    Config, Filepath, GroupBy, K8sConfig, LocalManifestLists, MetaManifest,
-    Selectors, ServerManifests,
+    Config, Filepath, GroupBy, K8sConfig, K8sResource, LocalManifestLists,
+    MetaManifest, Selectors, ServerManifests,
 )
 
 # Convenience: global logger instance to avoid repetitive code.
@@ -155,7 +155,7 @@ def unpack_list(manifest_list: dict,
         logit.error(f"Kind {kind} is not a list")
         return ({}, True)
 
-    # Strip of the "List".
+    # Strip off the "List" suffix from eg "DeploymentList".
     kind = kind[:-4]
 
     # Convenience.
@@ -455,7 +455,7 @@ def strip(
     manifest: dict,
     manifest_filters: Dict[str, list],
 ) -> Tuple[DotDict, dict, bool]:
-    """Strip `manifest` according to the filters in `square.schemas`.
+    """Strip `manifest` according to the filters in `square.cfgfile`.
 
     Inputs:
         k8sconfig: K8sConfig
@@ -528,9 +528,20 @@ def strip(
         # Remove all empty sub-dictionaries from `removed`.
         return {k: v for k, v in removed.items() if v != {}}
 
-    # Get the filters for the current resource. Use the default one if none exists.
-    filters = manifest_filters.get(manifest["kind"], square.schemas.default())
-    if not square.schemas.valid(filters):
+    # Look up the filters for the current resource in the following order:
+    # 1) Supplied `filters`
+    # 2) Square default filters for this resource `kind`.
+    # 3) Square default filters that apply to all resource kinds.
+    # Pick the first one that matches.
+    kind = manifest["kind"]
+    filters = manifest_filters.get(
+        kind,
+        square.DEFAULT_CONFIG.filters.get(
+            kind,
+            square.DEFAULT_CONFIG.filters["_common_"]
+        )
+    )
+    if not square.cfgfile.valid(filters):
         return ret_err
 
     # Remove the keys from the `manifest` according to `filters`.
@@ -934,14 +945,14 @@ def download(config: Config, k8sconfig: K8sConfig) -> Tuple[ServerManifests, boo
         for kind in sorted(config.selectors.kinds):
             # Get the K8s URL for the current resource kind. Ignore this
             # resource if K8s does not know about it. The reason for that could
-            # by a typo or that it is a custom resource that does not (yet) exist.
+            # be a typo or that it is a Custom Resource that does not (yet) exist.
             resource, err = square.k8s.resource(k8sconfig, MetaManifest("", kind, namespace, ""))  # noqa
             if err:
                 logit.warning(f"Skipping unknown resource <{kind}>")
                 continue
 
             try:
-                # Download the resource list from K8s.
+                # Download the resource manifests for the current `kind` from K8s.
                 manifest_list, err = square.k8s.get(k8sconfig.client, resource.url)
                 assert not err and manifest_list is not None
 
@@ -958,10 +969,10 @@ def download(config: Config, k8sconfig: K8sConfig) -> Tuple[ServerManifests, boo
                 err = any((v[2] for v in ret.values()))
                 assert not err
 
-                # Unpack the stripped manifests from the `strip` response.
-                # The "if v[0] is not None" is to satisfy MyPy - we already
-                # know they are not None or otherwise the previous assert would
-                # have failed.
+                # Unpack the stripped manifests from the `strip` response. The
+                # "if v[0] is not None" statement exists to satisfy MyPy - we
+                # already know they are not None or otherwise the previous
+                # assert would have failed.
                 manifests = {k: v[0] for k, v in ret.items() if v[0] is not None}
             except AssertionError:
                 # Return nothing, even if we had downloaded other kinds already.
@@ -970,3 +981,28 @@ def download(config: Config, k8sconfig: K8sConfig) -> Tuple[ServerManifests, boo
                 # Copy the manifests into the output dictionary.
                 server_manifests.update(manifests)
     return (server_manifests, False)
+
+
+def download_single(k8sconfig: K8sConfig,
+                    resource: K8sResource) -> Tuple[MetaManifest, dict, bool]:
+    """Similar to `download(...)` but only for a single Kubernetes `resource`.
+
+    Inputs:
+        k8sconfig: K8sConfig
+        resource: K8sResource
+
+    Returns:
+        MetaManifest, manifest: the K8s (meta)manifest.
+
+    """
+    try:
+        # Download the resource.
+        manifest, err = square.k8s.get(k8sconfig.client, resource.url)
+        assert not err
+
+        manifest, _, err = strip(k8sconfig, manifest, {})
+        assert not err
+    except AssertionError:
+        return (MetaManifest("", "", "", ""), {}, True)
+
+    return (make_meta(manifest), manifest, False)
