@@ -3,70 +3,25 @@ import json
 import logging
 import re
 from collections import Counter
-from types import SimpleNamespace
-from typing import Collection, Optional, Set, Tuple
+from typing import Collection, List, Optional, Set, Tuple
 
 import colorama
 import jsonpatch
-import pydantic
+import square.dotdict as dotdict
 import square.k8s as k8s
 import square.manio as manio
-import square.schemas
 import yaml
 from colorlog import ColoredFormatter
 from square.dtypes import (
     Config, DeltaCreate, DeltaDelete, DeltaPatch, DeploymentPlan,
-    DeploymentPlanMeta, Filepath, JsonPatch, K8sConfig, MetaManifest,
-    Selectors, ServerManifests,
+    DeploymentPlanMeta, JsonPatch, K8sConfig, MetaManifest, Selectors,
+    ServerManifests,
 )
 
 # Convenience: global logger instance to avoid repetitive code.
 logit = logging.getLogger("square")
 
 
-def load_config(fname: Filepath) -> Tuple[Config, bool]:
-    """Parse the Square configuration file `fname` and return it as a `Config`."""
-    err_resp = Config(Filepath(""), kubeconfig=Filepath(""), kubecontext=None), True
-    fname = Filepath(fname)
-
-    # Load the configuration file.
-    try:
-        raw = yaml.safe_load(Filepath(fname).read_text())
-    except FileNotFoundError as e:
-        logit.error(f"Cannot load config file <{fname}>: {e.args[1]}")
-        return err_resp
-    except yaml.YAMLError as exc:
-        msg = f"Could not parse YAML file {fname}"
-
-        # Special case: parser supplied location information.
-        mark = getattr(exc, "problem_mark", SimpleNamespace(line=-1, column=-1))
-        line, col = (mark.line + 1, mark.column + 1)
-        msg = f"YAML format error in {fname}: Line {line} Column {col}"
-        logit.error(msg)
-        return err_resp
-
-    if not isinstance(raw, dict):
-        logit.error(f"Config file <{fname}> has invalid structure")
-        return err_resp
-
-    # Parse the configuration into `ConfigFile` structure.
-    try:
-        cfg = Config(**raw)
-    except (pydantic.ValidationError, TypeError) as e:
-        logit.error(f"Schema is invalid: {e}")
-        return err_resp
-
-    # Remove the "_common_" filter and merge it into all the other filters.
-    common = cfg.filters.pop("_common_", [])
-    cfg.filters = {k: square.schemas.merge(common, v) for k, v in cfg.filters.items()}
-
-    # Make the necessary adjustments where the values in the file do not
-    # translate verbatim.
-    cfg.folder = fname.parent.absolute() / cfg.folder
-    cfg.selectors.kinds = set(cfg.selectors.kinds)
-    return cfg, False
-
-
 def translate_resource_kinds(cfg: Config, k8sconfig: K8sConfig) -> Config:
     """Convert `cfg.Selectors.kind` and `cfg.priorities` to their canonical names.
 
@@ -190,53 +145,78 @@ def partition_manifests(
     return (plan, False)
 
 
-def preferred_api(k8sconfig: K8sConfig,
-                  local: ServerManifests) -> Tuple[ServerManifests, bool]:
-    """Return a new version of `local` where all APIs point to the preferred group.
+def match_api_version(
+        k8sconfig: K8sConfig,
+        local: ServerManifests,
+        server: ServerManifests) -> Tuple[ServerManifests, bool]:
+    """Fetch the manifests from the endpoints defined in the local manifest.
 
-    Example:
-      in = {
-        MetaManifest(apiVersion='extensions/v1beta1', kind='Ingress', ...): {
-          'apiVersion': 'networking.k8s.io/v1beta1',
-          ...
-        }
-      }
-      out = {
-        MetaManifest(apiVersion='networking.k8s.io/v1beta1', kind='Ingress', ...): {
-          'apiVersion': 'networking.k8s.io/v1beta1',
-          ...
-        }
-      }
+    If a local manifest uses a different value for `apiVersion` then we need to
+    re-fetch those manifests from K8s via that endpoint. This function does
+    just that.
+
+    This function returns `server` verbatim if there is no overlap with
+    `server` and `local`.
+
+    Inputs:
+        config: Square configuration.
+        k8sconfig: K8sConfig
+        local: ServerManifests
+            Should be output from `load_manifest` or `load`.
+        server: ServerManifests
+            Should be output from `manio.download`.
+
+    Returns:
+        `server` but possibly with some entries re-fetched from the same K8s
+        endpoint that the equivalent resource in `local` specifies.
 
     """
-    # Check the API version in each MetaManifest and update it to the preferred
-    # version in both the MetaManifest and the associated K8s YAML manifest.
-    out = {}
-    for meta, manifest in local.items():
-        # The current `meta` can be outdated but must still be a valid resource
-        # in the current cluster.
-        if k8s.resource(k8sconfig, meta)[1]:
-            return {}, True
+    # Avoid side effects.
+    server = copy.deepcopy(server)
 
-        # Get the canonical resource description.
-        res, err = k8s.resource(k8sconfig, meta._replace(apiVersion=""))
-        if err:
-            logit.critical(f"BUG: resource <{meta}> should have been valid")
-            return {}, True
+    # Find the resources that exist in local manifests and on K8s. The
+    # resources are identical if their MetaManifest are identical save for the
+    # `apiVersion` field.
+    mm_loc = {meta._replace(apiVersion=""): meta for meta in local}
+    mm_srv = {meta._replace(apiVersion=""): meta for meta in server}
+    meta_overlap = set(mm_loc.keys()) & set(mm_srv.keys())
 
-        # Log a warning if the manifest uses an outdated API group.
-        if meta.apiVersion != res.apiVersion:
+    # Iterate over all the resources that exist on both the server and locally,
+    # even though they may use different API versions.
+    to_download: List[MetaManifest] = []
+    for meta in meta_overlap:
+        # Lookup the full MetaManifest for the local and server resource.
+        # NOTE: meta_{loc,srv} are identical except possibly for the `apiVersion` field.
+        meta_loc = mm_loc[meta]
+        meta_srv = mm_srv[meta]
+
+        # Do nothing if the `apiVersions` match because we can already compute a
+        # plan for it. However, if the `apiVersions` differ then we will
+        # replace entry in `server` with the one fetched from the correct K8s
+        # endpoint (see next section).
+        if meta_loc != meta_srv:
+            del server[meta_srv]
+            to_download.append(meta_loc)
             logit.warning(
-                f"Switching <{res.kind} {meta.namespace}/{meta.name}>"
-                f" from <{meta.apiVersion}> to <{res.apiVersion}>"
-                " - Please update your manifests."
+                f"Using non-default {meta.kind.upper()} endpoint "
+                f"<{meta_loc.apiVersion}>"
             )
 
-            # Update the API version for the resource to whatever K8s prefers.
-            meta = meta._replace(apiVersion=res.apiVersion)
-            manifest["apiVersion"] = meta.apiVersion
-        out[meta] = manifest
-    return out, False
+    # Re-fetch the resources we already got but this time from the correct endpoint.
+    for meta in to_download:
+        # Construct the correct K8sResource.
+        resource, err = k8s.resource(k8sconfig, meta)
+        assert not err
+
+        # Download all resources of the current kind.
+        meta, manifest, err = manio.download_single(k8sconfig, resource)
+        assert not err
+
+        # Add the resource to the `server` dict. This will have been one of
+        # those we deleted a few lines earlier.
+        server[meta] = manifest
+
+    return server, False
 
 
 def compile_plan(
@@ -244,7 +224,7 @@ def compile_plan(
         k8sconfig: K8sConfig,
         local: ServerManifests,
         server: ServerManifests) -> Tuple[DeploymentPlan, bool]:
-    """Return the `DeploymentPlan` to transition K8s to state of `local`.
+    """Return the `DeploymentPlan` to transition K8s to the `local` state.
 
     The deployment plan is a named tuple. It specifies which resources to
     create, patch and delete to ensure that the state of K8s matches that
@@ -264,6 +244,15 @@ def compile_plan(
     """
     err_resp = (DeploymentPlan(tuple(), tuple(), tuple()), True)
 
+    # Abort unless all local manifests reference valid K8s resource kinds.
+    if any([k8s.resource(k8sconfig, meta)[1] for meta in local]):
+        return err_resp
+
+    # Replace the server resources fetched from the K8s preferred endpoint with
+    # those fetched from the endpoint used by the local manifest.
+    server, err = match_api_version(k8sconfig, local, server)
+    assert not err
+
     # Apply the filters to all local and server manifests before we compute patches.
     server = {
         meta: manio.strip(k8sconfig, man, config.filters)
@@ -281,16 +270,10 @@ def compile_plan(
         logit.error("Could not strip all manifests.")
         return err_resp
 
-    # Unpack the stripped manifests (`manio.strip` returned a tuple with three
-    # entries, and here we unpack the first one because it is the stripped manifest).
-    server = {k: square.dotdict.undo(v[0]) for k, v in server.items()}
-    local = {k: square.dotdict.undo(v[0]) for k, v in local.items()}
-
-    # Replace the API group of the local resource with the one K8s prefers.
-    local, err = preferred_api(k8sconfig, local)
-    if err:
-        logit.error("Could not determine the preferred APIs for all manifests.")
-        return err_resp
+    # Unpack the stripped manifests (first element in the tuple returned from
+    # `manio.strip`).
+    server = {k: dotdict.undo(v[0]) for k, v in server.items()}
+    local = {k: dotdict.undo(v[0]) for k, v in local.items()}
 
     # Partition the set of meta manifests into create/delete/patch groups.
     plan, err = partition_manifests(local, server)
@@ -714,16 +697,21 @@ def get_resources(cfg: Config) -> bool:
         load_selectors = Selectors(kinds=k8sconfig.kinds, labels=[], namespaces=[])
 
         # Load manifests from local files.
-        _, local, err = manio.load(cfg.folder, load_selectors)
+        local_meta, local_path, err = manio.load(cfg.folder, load_selectors)
         assert not err
 
         # Download manifests from K8s.
         server, err = manio.download(cfg, k8sconfig)
         assert not err
 
+        # Replace the server resources fetched from K8s' preferred endpoint with
+        # the one from the endpoint referenced in the local manifest.
+        server, err = match_api_version(k8sconfig, local_meta, server)
+        assert not err
+
         # Sync the server manifests into the local manifests. All this happens in
         # memory and no files will be modified here - see `manio.save` in the next step.
-        synced_manifests, err = manio.sync(local, server, cfg.selectors, cfg.groupby)
+        synced_manifests, err = manio.sync(local_path, server, cfg.selectors, cfg.groupby)
         assert not err
 
         # Write the new manifest files.
